/***************************************************************************
 *   Copyright (C) 2006 by George Kiagiadakis   *
 *   gkiagia@users.sourceforge.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "configuration.h"
#include <QMessageBox>
#include <QProcess>
#include <QSplashScreen>
#include <QApplication>

Configuration::Configuration(const QString &path, QString name)
	:QDir(path)
{
	cfgName = name;
	cfgSettings = new QSettings( absoluteFilePath("qtwine.ini"), QSettings::IniFormat ); // configuration specific settings

	if ( QFileInfo(absolutePath()).isSymLink() )
		symlink = TRUE;
	else
		symlink = FALSE;
}

Configuration::~Configuration() {
	delete cfgSettings;
}

void Configuration::renameSelf(QString newName)
{
	if ( cfgName == "Wine Default" )
		return;

	cdUp();
	if ( !rename(cfgName, newName) )
	{
		QMessageBox::critical(0, "Error", "Could not rename configuration.\nCheck its permissions and/or if a configuration with the new name already exists", "OK");
		cd(cfgName);
		return;
	}
	cd(newName);
	fixDriveC();
	cfgName = newName;
}

void Configuration::fixDriveC()
{
	cd("dosdevices");
	remove("c:");
	QFile::link("../drive_c", filePath("c:") );
	cdUp();
}

bool Configuration::rmSelf()
{
	if ( symlink == TRUE )
	{
		QString dir = dirName();
		cdUp();
		return remove(dir);
	}
	else
	{
		if ( !recursiveRmDir( *this ) )
		{
			QMessageBox::critical(0, "Error", "Could not remove configuration. Check its permissions", "OK");
			return false;
		}
		return true;
	}
}

bool Configuration::recursiveRmDir(QDir dir) // works like `rm -r`
{
	int i;
	QFileInfoList list = dir.entryInfoList( QDir::AllEntries | QDir::NoDotAndDotDot | QDir::Hidden | QDir::System | QDir::CaseSensitive, QDir::DirsLast );
	for (i=0; i < list.count() ; i++)
	{
		if ( list[i].isSymLink() )
		{
			if ( !dir.remove(list[i].fileName()) ) return FALSE;
		}
		else if ( list[i].isDir() )
		{
			if ( !recursiveRmDir( QDir(list[i].absoluteFilePath()) ) ) return FALSE;
		}
		else
		{
			if ( !dir.remove(list[i].fileName()) ) return FALSE;
		}
	}
	QString dirname = dir.dirName();
	dir.cdUp();
	return dir.rmdir(dirname);
}


QString Configuration::name() const
{
	return cfgName;
}

bool Configuration::isSymlink() const
{
	return symlink;
}

QString Configuration::description() // TODO in 0.4
{
	return cfgSettings->value("description", QString()).toString();
}

void Configuration::setDescription(QString d)
{
	cfgSettings->setValue("description", d);
}

bool Configuration::update() // TODO in 0.4
{
	QProcess wineprefixcreate;
	wineprefixcreate.setReadChannelMode(QProcess::MergedChannels);
	wineprefixcreate.start("wineprefixcreate", QStringList() << "--prefix" << absolutePath());

	QSplashScreen ss(QPixmap(QString(":/graphics/images/wait.png"))); // FIXME this appears in the center of the screen
	ss.show();
	qApp->processEvents();

	wineprefixcreate.waitForFinished(120000); // 2 min

	ss.hide();
	qApp->processEvents();

	if ( wineprefixcreate.exitStatus() == QProcess::NormalExit ) {
		if ( wineprefixcreate.exitCode() == 0 ) {
			QMessageBox::information(0, "Success", "This wine configuration was updated successfully.\nwineprefixcreate exit code = 0", "OK");
			return TRUE;
		} else {
			QMessageBox::information(0, "Failure", QString("This wine configuration was NOT updated successfully.\nwineprefixcreate exit code = %1\n\nStandard output / Standard error:\n%2").arg(wineprefixcreate.exitCode()).arg( QString(wineprefixcreate.readAllStandardOutput()) ), "OK");
		}
	} else {
		QMessageBox::information(0, "Failure", QString("This wine configuration was NOT updated successfully.\nwineprefixcreate crashed\n\nStandard output / Standard error:\n%1").arg( QString(wineprefixcreate.readAllStandardOutput()) ), "OK");
	}

	return FALSE;
}

void Configuration::personalize() // This is called after a duplicate operation, just to fix some details
{
	fixDriveC();
	// TODO update qtwine.ini if it contains personal information // to version 0.4
}
